/**
 * @file libgalago-gtk/galago-gtk-account-combo-box.c Account ComboBox widget
 *
 * @Copyright (C) 2004-2005 Christian Hammond.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#include <libgalago-gtk/galago-gtk-account-combo-box.h>
#include <libgalago-gtk/galago-gdk-pixbuf.h>
#include <libgalago/galago.h>
#include <string.h>

#if GTK_CHECK_VERSION(2,4,0)

struct _GalagoGtkAccountComboBoxPriv
{
	GalagoPerson *person;
	GalagoService *service;

	galago_bool me_person_set;

	gboolean me_accounts;

	GtkWidget *service_combobox;

	gint service_changed_sig_id;
	gint service_combo_box_destroyed_sig_id;

	unsigned long account_added_id;
	unsigned long account_removed_id;
	unsigned long account_connected_id;
	unsigned long account_disconnected_id;

	unsigned long person_added_id;
	unsigned long person_destroy_id;
};

enum
{
	ACCOUNT_CHANGED,
	LAST_SIGNAL
};

enum
{
	COLUMN_DATA,
	COLUMN_ICON,
	COLUMN_NAME,
	NUM_COLUMNS
};

static void galago_gtk_account_combo_box_class_init(
	GalagoGtkAccountComboBoxClass *klass);
static void galago_gtk_account_combo_box_init(
	GalagoGtkAccountComboBox *combobox);
static void galago_gtk_account_combo_box_finalize(GObject *obj);
static void galago_gtk_account_combo_box_destroy(GtkObject *obj);
static void account_added_cb(void *obj, const char *property,
							 GalagoAccount *account,
							 GalagoGtkAccountComboBox *combobox);
static void account_removed_cb(void *obj, const char *property,
							   GalagoAccount *account,
							   GalagoGtkAccountComboBox *combobox);
static void account_conn_changed_cb(GalagoAccount *account,
									const char *property,
									GalagoGtkAccountComboBox *combobox);
static void person_destroy_cb(GalagoPerson *person, const char *property,
							  GalagoGtkAccountComboBox *combobox);

static GtkComboBoxClass *parent_class = NULL;
static guint signals[LAST_SIGNAL] = {0};

GType
galago_gtk_account_combo_box_get_type(void)
{
	static GType type = 0;

	if (!type)
	{
		static const GTypeInfo info =
		{
			sizeof(GalagoGtkAccountComboBoxClass),
			NULL,
			NULL,
			(GClassInitFunc)galago_gtk_account_combo_box_class_init,
			NULL,
			NULL,
			sizeof(GalagoGtkAccountComboBox),
			0,
			(GInstanceInitFunc)galago_gtk_account_combo_box_init
		};

		type = g_type_register_static(GTK_TYPE_COMBO_BOX,
									  "GalagoGtkAccountComboBox", &info, 0);
	}

	return type;
}

static void
galago_gtk_account_combo_box_class_init(GalagoGtkAccountComboBoxClass *klass)
{
	GObjectClass *gobject_class;
	GtkObjectClass *object_class;

	parent_class = g_type_class_peek_parent(klass);

	gobject_class = G_OBJECT_CLASS(klass);
	object_class  = GTK_OBJECT_CLASS(klass);

	gobject_class->finalize = galago_gtk_account_combo_box_finalize;

	object_class->destroy = galago_gtk_account_combo_box_destroy;

	signals[ACCOUNT_CHANGED] =
		g_signal_new("account_changed",
					 G_TYPE_FROM_CLASS(gobject_class),
					 G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
					 G_STRUCT_OFFSET(GalagoGtkAccountComboBoxClass,
									 account_changed),
					 NULL, NULL,
					 gtk_marshal_VOID__POINTER,
					 G_TYPE_NONE, 1,
					 G_TYPE_POINTER);
}

static void
combo_box_changed(GalagoGtkAccountComboBox *combobox, gpointer user_data)
{
	GalagoAccount *account;

	account = galago_gtk_account_combo_box_get_active(combobox);

	g_signal_emit(combobox, signals[ACCOUNT_CHANGED], 0, account);
}

static void
galago_gtk_account_combo_box_init(GalagoGtkAccountComboBox *combobox)
{
	GtkListStore *store;
	GtkCellRenderer *cell;

	combobox->priv = g_new0(GalagoGtkAccountComboBoxPriv, 1);

	store = gtk_list_store_new(NUM_COLUMNS, G_TYPE_POINTER,
							   GDK_TYPE_PIXBUF, G_TYPE_STRING);

	gtk_combo_box_set_model(GTK_COMBO_BOX(combobox),
							GTK_TREE_MODEL(store));

	g_object_unref(G_OBJECT(store));

	cell = gtk_cell_renderer_pixbuf_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(combobox), cell, FALSE);
	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(combobox), cell,
								   "pixbuf", COLUMN_ICON,
								   NULL);

	cell = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(combobox), cell, TRUE);
	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(combobox), cell,
								   "text", COLUMN_NAME,
								   NULL);

	g_signal_connect(G_OBJECT(combobox), "changed",
					 G_CALLBACK(combo_box_changed), NULL);

	combobox->priv->account_added_id =
		galago_signal_connect_class(GALAGO_CLASS_SERVICE, "account-added",
									GALAGO_CALLBACK(account_added_cb),
									combobox);
	combobox->priv->account_removed_id =
		galago_signal_connect_class(GALAGO_CLASS_SERVICE, "account-removed",
									GALAGO_CALLBACK(account_removed_cb),
									combobox);

	combobox->priv->account_connected_id =
		galago_signal_connect_class(GALAGO_CLASS_ACCOUNT, "connected",
									GALAGO_CALLBACK(account_conn_changed_cb),
									combobox);

	combobox->priv->account_disconnected_id =
		galago_signal_connect_class(GALAGO_CLASS_ACCOUNT, "disconnected",
									GALAGO_CALLBACK(account_conn_changed_cb),
									combobox);
}

static void
galago_gtk_account_combo_box_finalize(GObject *obj)
{
	GalagoGtkAccountComboBox *combobox;

	g_return_if_fail(obj != NULL);
	g_return_if_fail(GALAGO_GTK_IS_ACCOUNT_COMBO_BOX(obj));

	combobox = GALAGO_GTK_ACCOUNT_COMBO_BOX(obj);

	g_free(combobox->priv);

	if (G_OBJECT_CLASS(parent_class)->finalize)
		G_OBJECT_CLASS(parent_class)->finalize(obj);
}

static void
galago_gtk_account_combo_box_destroy(GtkObject *obj)
{
	GalagoGtkAccountComboBox *combobox;

	g_return_if_fail(obj != NULL);
	g_return_if_fail(GALAGO_GTK_IS_ACCOUNT_COMBO_BOX(obj));

	combobox = GALAGO_GTK_ACCOUNT_COMBO_BOX(obj);

	if (combobox->priv->service_changed_sig_id != 0)
	{
		g_signal_handler_disconnect(G_OBJECT(combobox->priv->service_combobox),
									combobox->priv->service_changed_sig_id);
		combobox->priv->service_changed_sig_id = 0;
	}

	if (combobox->priv->service_combo_box_destroyed_sig_id != 0)
	{
		g_signal_handler_disconnect(G_OBJECT(combobox->priv->service_combobox),
			combobox->priv->service_combo_box_destroyed_sig_id);
		combobox->priv->service_combo_box_destroyed_sig_id = 0;
	}

	if (combobox->priv->account_added_id > 0)
		galago_signals_disconnect_by_id(combobox->priv->account_added_id);

	if (combobox->priv->account_removed_id > 0)
		galago_signals_disconnect_by_id(combobox->priv->account_removed_id);

	if (combobox->priv->account_connected_id > 0)
		galago_signals_disconnect_by_id(combobox->priv->account_connected_id);

	if (combobox->priv->account_disconnected_id > 0)
		galago_signals_disconnect_by_id(combobox->priv->account_disconnected_id);

	if (combobox->priv->person_added_id > 0)
	{
		galago_signals_disconnect_by_id(combobox->priv->person_added_id);
		combobox->priv->person_added_id = 0;
	}

	if (combobox->priv->person_destroy_id > 0)
	{
		galago_signals_disconnect_by_id(combobox->priv->person_destroy_id);
		combobox->priv->person_destroy_id = 0;
	}

	combobox->priv->account_added_id        = 0;
	combobox->priv->account_removed_id      = 0;
	combobox->priv->account_connected_id    = 0;
	combobox->priv->account_disconnected_id = 0;

	if (GTK_OBJECT_CLASS(parent_class)->destroy)
		GTK_OBJECT_CLASS(parent_class)->destroy(obj);
}

static void
append_account(GtkListStore *store, GalagoAccount *account)
{
	GdkPixbuf *pixbuf;
	GtkTreeIter iter;

	pixbuf = galago_gdk_pixbuf_new_from_account_with_size(account,
		GTK_ICON_SIZE_MENU);

	gtk_list_store_append(store, &iter);
	gtk_list_store_set(store, &iter,
					   COLUMN_DATA, account,
					   COLUMN_ICON, pixbuf,
					   COLUMN_NAME, galago_account_get_display_name(account),
					   -1);

	if (pixbuf != NULL)
		g_object_unref(G_OBJECT(pixbuf));
}

static void
append_accounts_from_person(GalagoGtkAccountComboBox *combobox,
							GtkListStore *store, GalagoPerson *person,
							GalagoService *service)
{
	const GalagoList *l;

	for (l = galago_person_get_accounts(person, TRUE);
		 l != NULL;
		 l = l->next)
	{
		GalagoAccount *account = (GalagoAccount *)l->data;

		if (service == NULL || galago_account_get_service(account) == service)
			append_account(store, account);
	}
}

static void
append_accounts_from_service(GalagoGtkAccountComboBox *combobox,
							 GtkListStore *store, GalagoService *service)
{
	const GalagoList *l;

	for (l = galago_service_get_accounts(service, TRUE);
		 l != NULL;
		 l = l->next)
	{
		append_account(store, (GalagoAccount *)l->data);
	}
}

static void
rebuild_list(GalagoGtkAccountComboBox *combobox)
{
	GtkListStore *store;

	store = (GtkListStore *)gtk_combo_box_get_model(GTK_COMBO_BOX(combobox));

	gtk_list_store_clear(store);

	if (combobox->priv->person != NULL)
	{
		append_accounts_from_person(combobox, store,
									combobox->priv->person,
									combobox->priv->service);
	}
	else if (combobox->priv->person == NULL &&
			 combobox->priv->me_person_set)
	{
		return;
	}
	else if (combobox->priv->service != NULL)
	{
		append_accounts_from_service(combobox, store,
									 combobox->priv->service);
	}
	else
	{
		const GalagoList *l;

		for (l = galago_core_get_services(FALSE, TRUE); l != NULL; l = l->next)
		{
			append_accounts_from_service(combobox, store,
										 (GalagoService *)l->data);
		}
	}

	gtk_combo_box_set_active(GTK_COMBO_BOX(combobox), 0);
}

static void
account_added_cb(void *obj, const char *property, GalagoAccount *account,
				 GalagoGtkAccountComboBox *combobox)
{
	GalagoAccount *cur_account;

	cur_account = galago_gtk_account_combo_box_get_active(combobox);

	rebuild_list(combobox);

	if (cur_account != NULL)
	{
		if (galago_gtk_account_combo_box_get_active(combobox) != cur_account)
			galago_gtk_account_combo_box_set_active(combobox, cur_account);
	}
	else
		gtk_combo_box_set_active(GTK_COMBO_BOX(combobox), 0);
}

static void
account_removed_cb(void *obj, const char *property, GalagoAccount *account,
				   GalagoGtkAccountComboBox *combobox)
{
	GalagoAccount *cur_account;

	cur_account = galago_gtk_account_combo_box_get_active(combobox);

	rebuild_list(combobox);

	if (cur_account == account)
	{
		if (galago_gtk_account_combo_box_get_active(combobox) != cur_account)
			galago_gtk_account_combo_box_set_active(combobox, cur_account);
	}
	else
		gtk_combo_box_set_active(GTK_COMBO_BOX(combobox), 0);
}

static void
account_conn_changed_cb(GalagoAccount *account, const char *property,
						GalagoGtkAccountComboBox *combobox)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	gboolean cont;

	model = gtk_combo_box_get_model(GTK_COMBO_BOX(combobox));

	for (cont = gtk_tree_model_get_iter_first(model, &iter);
		 cont;
		 cont = gtk_tree_model_iter_next(model, &iter))
	{
		GalagoAccount *temp_account;

		gtk_tree_model_get(model, &iter,
						   COLUMN_DATA, &temp_account,
						   -1);

		if (temp_account == account)
		{
			GdkPixbuf *pixbuf;

			pixbuf = galago_gdk_pixbuf_new_from_account_with_size(account,
				GTK_ICON_SIZE_MENU);

			gtk_list_store_set(GTK_LIST_STORE(model), &iter,
							   COLUMN_ICON, pixbuf,
							   -1);

			if (pixbuf != NULL)
				g_object_unref(G_OBJECT(pixbuf));

			break;
		}
	}
}

static void
person_added_cb(void *obj, const char *property, GalagoPerson *person,
				GalagoGtkAccountComboBox *combobox)
{
	if (combobox->priv->me_person_set && galago_person_is_me(person))
		galago_gtk_account_combo_box_set_person(combobox, person);
}

static void
person_destroy_cb(GalagoPerson *person, const char *property,
				  GalagoGtkAccountComboBox *combobox)
{
	combobox->priv->person = NULL;

	galago_signals_disconnect_by_id(combobox->priv->person_destroy_id);
	combobox->priv->person_destroy_id = 0;

	if (galago_person_is_me(person))
	{
		combobox->priv->person_added_id =
			galago_signal_connect(galago_get_core(), "person-added",
								  GALAGO_CALLBACK(person_added_cb),
								  combobox);
	}
}

GtkWidget *
galago_gtk_account_combo_box_new(gboolean me_accounts)
{
	GalagoGtkAccountComboBox *combobox;

	combobox = g_object_new(GALAGO_GTK_TYPE_ACCOUNT_COMBO_BOX, NULL);

	combobox->priv->me_accounts = me_accounts;

	if (me_accounts)
	{
		galago_gtk_account_combo_box_set_person(combobox,
			galago_core_get_me(FALSE, TRUE));
	}

	rebuild_list(combobox);

	gtk_combo_box_set_active(GTK_COMBO_BOX(combobox), 0);

	return GTK_WIDGET(combobox);
}

void
galago_gtk_account_combo_box_set_person(GalagoGtkAccountComboBox *combobox,
										GalagoPerson *person)
{
	g_return_if_fail(combobox != NULL);
	g_return_if_fail(GALAGO_GTK_IS_ACCOUNT_COMBO_BOX(combobox));

	if (combobox->priv->person == person)
		return;

	if (combobox->priv->person_added_id > 0)
	{
		galago_signals_disconnect_by_id(combobox->priv->person_added_id);
		combobox->priv->person_added_id = 0;
	}

	if (combobox->priv->person_destroy_id > 0)
	{
		galago_signals_disconnect_by_id(combobox->priv->person_destroy_id);
		combobox->priv->person_destroy_id = 0;
	}

	combobox->priv->person = person;

	if (person != NULL)
	{
		combobox->priv->me_person_set = galago_person_is_me(person);

		combobox->priv->person_destroy_id =
			galago_signal_connect(person, "destroy",
								  GALAGO_CALLBACK(person_destroy_cb),
								  combobox);
	}
	else
		combobox->priv->me_person_set = FALSE;

	rebuild_list(combobox);
}

GalagoPerson *
galago_gtk_account_combo_box_get_person(
	const GalagoGtkAccountComboBox *combobox)
{
	g_return_val_if_fail(combobox != NULL,                          NULL);
	g_return_val_if_fail(GALAGO_GTK_IS_ACCOUNT_COMBO_BOX(combobox), NULL);

	return combobox->priv->person;
}

void
galago_gtk_account_combo_box_set_service(GalagoGtkAccountComboBox *combobox,
										 GalagoService *service)
{
	g_return_if_fail(combobox != NULL);
	g_return_if_fail(GALAGO_GTK_IS_ACCOUNT_COMBO_BOX(combobox));

	if (combobox->priv->service == service)
		return;

	if (combobox->priv->account_added_id > 0)
	{
		galago_signals_disconnect_by_id(combobox->priv->account_added_id);
		combobox->priv->account_added_id = 0;
	}

	if (combobox->priv->account_removed_id > 0)
	{
		galago_signals_disconnect_by_id(combobox->priv->account_removed_id);
		combobox->priv->account_removed_id = 0;
	}

	combobox->priv->service = service;

	if (service == NULL)
	{
		combobox->priv->account_added_id =
			galago_signal_connect_class(GALAGO_CLASS_SERVICE, "account-added",
										GALAGO_CALLBACK(account_added_cb),
										combobox);
		combobox->priv->account_removed_id =
			galago_signal_connect_class(GALAGO_CLASS_SERVICE,
										"account-removed",
										GALAGO_CALLBACK(account_removed_cb),
										combobox);
	}
	else
	{
		combobox->priv->account_added_id =
			galago_signal_connect(service, "account-added",
								  GALAGO_CALLBACK(account_added_cb),
								  combobox);
		combobox->priv->account_removed_id =
			galago_signal_connect(service, "account-removed",
								  GALAGO_CALLBACK(account_removed_cb),
								  combobox);
	}

	rebuild_list(combobox);
}

GalagoService *
galago_gtk_account_combo_box_get_service(
	const GalagoGtkAccountComboBox *combobox)
{
	g_return_val_if_fail(combobox != NULL,                          NULL);
	g_return_val_if_fail(GALAGO_GTK_IS_ACCOUNT_COMBO_BOX(combobox), NULL);

	return combobox->priv->service;
}

void
galago_gtk_account_combo_box_set_active(GalagoGtkAccountComboBox *combobox,
										const GalagoAccount *account)
{
	GtkTreeModel *model;
	GtkTreeIter iter;
	gboolean cont;

	g_return_if_fail(combobox != NULL);
	g_return_if_fail(GALAGO_GTK_IS_ACCOUNT_COMBO_BOX(combobox));
	g_return_if_fail(account != NULL);

	model = gtk_combo_box_get_model(GTK_COMBO_BOX(combobox));

	for (cont = gtk_tree_model_get_iter_first(model, &iter);
		 cont;
		 cont = gtk_tree_model_iter_next(model, &iter))
	{
		GalagoAccount *temp_account;

		gtk_tree_model_get(model, &iter,
						   COLUMN_DATA, &temp_account,
						   -1);

		if (temp_account == account)
			gtk_combo_box_set_active_iter(GTK_COMBO_BOX(combobox), &iter);
	}
}

GalagoAccount *
galago_gtk_account_combo_box_get_active(
	const GalagoGtkAccountComboBox *combobox)
{
	GalagoAccount *account = NULL;
	GtkTreeModel *model;
	GtkTreeIter iter;

	g_return_val_if_fail(combobox != NULL,                          NULL);
	g_return_val_if_fail(GALAGO_GTK_IS_ACCOUNT_COMBO_BOX(combobox), NULL);

	model = gtk_combo_box_get_model(GTK_COMBO_BOX(combobox));

	if (gtk_combo_box_get_active_iter(GTK_COMBO_BOX(combobox), &iter))
	{
		gtk_tree_model_get(model, &iter,
						   COLUMN_DATA, &account,
						   -1);
	}

	return account;
}

static void
service_changed_cb(GalagoGtkServiceComboBox *service_combobox,
				   GalagoService *service, GalagoGtkAccountComboBox *combobox)
{
	galago_gtk_account_combo_box_set_service(combobox,
		galago_gtk_service_combo_box_get_active(service_combobox));
}

static void
service_combo_box_destroyed_cb(GalagoGtkServiceComboBox *service_combobox,
							   GalagoGtkAccountComboBox *combobox)
{
	combobox->priv->service_changed_sig_id             = 0;
	combobox->priv->service_combo_box_destroyed_sig_id = 0;
	combobox->priv->service_combobox                   = NULL;

	galago_gtk_account_combo_box_set_service(combobox, NULL);
}

void
galago_gtk_account_combo_box_set_services_combo_box(
	GalagoGtkAccountComboBox *combobox, GtkWidget *service_combobox)
{
	GalagoService *service = NULL;

	g_return_if_fail(combobox != NULL);
	g_return_if_fail(GALAGO_GTK_IS_ACCOUNT_COMBO_BOX(combobox));
	g_return_if_fail(service_combobox == NULL ||
					 GALAGO_GTK_IS_SERVICE_COMBO_BOX(service_combobox));

	if (service_combobox == combobox->priv->service_combobox)
		return;

	if (combobox->priv->service_changed_sig_id != 0)
	{
		g_signal_handler_disconnect(
			G_OBJECT(combobox->priv->service_combobox),
			combobox->priv->service_changed_sig_id);
	}

	if (combobox->priv->service_combo_box_destroyed_sig_id != 0)
	{
		g_signal_handler_disconnect(
			G_OBJECT(combobox->priv->service_combobox),
			combobox->priv->service_combo_box_destroyed_sig_id);
	}

	combobox->priv->service_combobox = service_combobox;

	if (service_combobox != NULL)
	{
		combobox->priv->service_changed_sig_id =
			g_signal_connect(G_OBJECT(service_combobox), "service_changed",
							 G_CALLBACK(service_changed_cb), combobox);
		combobox->priv->service_combo_box_destroyed_sig_id =
			g_signal_connect(G_OBJECT(service_combobox), "destroy",
							 G_CALLBACK(service_combo_box_destroyed_cb),
							 combobox);

		service = galago_gtk_service_combo_box_get_active(
			GALAGO_GTK_SERVICE_COMBO_BOX(service_combobox));
	}

	galago_gtk_account_combo_box_set_service(combobox, service);
}

GtkWidget *
galago_gtk_account_combo_box_get_services_combo_box(
	const GalagoGtkAccountComboBox *combobox)
{
	g_return_val_if_fail(combobox != NULL,                          NULL);
	g_return_val_if_fail(GALAGO_GTK_IS_ACCOUNT_COMBO_BOX(combobox), NULL);

	return combobox->priv->service_combobox;
}

#endif /* GTK_CHECK_VERSION(2,4,0) */
